@@ -23,9 +23,11 @@
 
 #include <boost/optional.hpp>
 #include <boost/variant.hpp>
+#include <cstdint>
 #include <functional>
 #include <list>
 #include <memory>
+#include <set>
 #include <utility>
 
 #include "BrokerConsumerStatsImpl.h"
@@ -66,7 +68,7 @@ class BrokerEntryMetadata;
 class MessageMetadata;
 }  // namespace proto
 
-enum ConsumerTopicType
+enum ConsumerTopicType : uint8_t
 {
     NonPartitioned,
     Partitioned
@@ -85,17 +87,17 @@ enum class SeekStatus : std::uint8_t
 
 class ConsumerImpl : public ConsumerImplBase {
    public:
-    ConsumerImpl(const ClientImplPtr client, const std::string& topic, const std::string& subscriptionName,
+    ConsumerImpl(const ClientImplPtr& client, const std::string& topic, const std::string& subscriptionName,
                  const ConsumerConfiguration&, bool isPersistent, const ConsumerInterceptorsPtr& interceptors,
-                 const ExecutorServicePtr listenerExecutor = ExecutorServicePtr(), bool hasParent = false,
+                 const ExecutorServicePtr& listenerExecutor = ExecutorServicePtr(), bool hasParent = false,
                  const ConsumerTopicType consumerTopicType = NonPartitioned,
                  Commands::SubscriptionMode = Commands::SubscriptionModeDurable,
-                 boost::optional<MessageId> startMessageId = boost::none);
+                 const boost::optional<MessageId>& startMessageId = boost::none);
     ~ConsumerImpl();
     void setPartitionIndex(int partitionIndex);
     int getPartitionIndex();
     void sendFlowPermitsToBroker(const ClientConnectionPtr& cnx, int numMessages);
-    uint64_t getConsumerId();
+    uint64_t getConsumerId() const noexcept { return consumerId_; }
     void messageReceived(const ClientConnectionPtr& cnx, const proto::CommandMessage& msg,
                          bool& isChecksumValid, proto::BrokerEntryMetadata& brokerEntryMetadata,
                          proto::MessageMetadata& msgMetadata, SharedBuffer& payload);
@@ -113,15 +115,20 @@ class ConsumerImpl : public ConsumerImplBase {
     const std::string& getTopic() const override;
     Result receive(Message& msg) override;
     Result receive(Message& msg, int timeout) override;
-    void receiveAsync(ReceiveCallback callback) override;
-    void unsubscribeAsync(ResultCallback callback) override;
-    void acknowledgeAsync(const MessageId& msgId, ResultCallback callback) override;
-    void acknowledgeAsync(const MessageIdList& messageIdList, ResultCallback callback) override;
-    void acknowledgeCumulativeAsync(const MessageId& msgId, ResultCallback callback) override;
-    void closeAsync(ResultCallback callback) override;
+    void receiveAsync(const ReceiveCallback& callback) override;
+    void unsubscribeAsync(const ResultCallback& callback) override;
+    void acknowledgeAsync(const MessageId& msgId, const ResultCallback& callback) override;
+    void acknowledgeAsync(const MessageIdList& messageIdList, const ResultCallback& callback) override;
+    void acknowledgeCumulativeAsync(const MessageId& msgId, const ResultCallback& callback) override;
+    void closeAsync(const ResultCallback& callback) override;
     void start() override;
     void shutdown() override;
+    void internalShutdown();
     bool isClosed() override;
+    bool isClosingOrClosed() const noexcept {
+        const auto state = state_.load(std::memory_order_relaxed);
+        return state == Closing || state == Closed;
+    }
     bool isOpen() override;
     Result pauseMessageListener() override;
     Result resumeMessageListener() override;
@@ -129,14 +136,14 @@ class ConsumerImpl : public ConsumerImplBase {
     void redeliverUnacknowledgedMessages(const std::set<MessageId>& messageIds) override;
     const std::string& getName() const override;
     int getNumOfPrefetchedMessages() const override;
-    void getBrokerConsumerStatsAsync(BrokerConsumerStatsCallback callback) override;
-    void getLastMessageIdAsync(BrokerGetLastMessageIdCallback callback) override;
-    void seekAsync(const MessageId& msgId, ResultCallback callback) override;
-    void seekAsync(uint64_t timestamp, ResultCallback callback) override;
+    void getBrokerConsumerStatsAsync(const BrokerConsumerStatsCallback& callback) override;
+    void getLastMessageIdAsync(const BrokerGetLastMessageIdCallback& callback) override;
+    void seekAsync(const MessageId& msgId, const ResultCallback& callback) override;
+    void seekAsync(uint64_t timestamp, const ResultCallback& callback) override;
     void negativeAcknowledge(const MessageId& msgId) override;
     bool isConnected() const override;
     uint64_t getNumberOfConnectedConsumer() override;
-    void hasMessageAvailableAsync(HasMessageAvailableCallback callback) override;
+    void hasMessageAvailableAsync(const HasMessageAvailableCallback& callback) override;
 
     virtual void disconnectConsumer();
     virtual void disconnectConsumer(const boost::optional<std::string>& assignedBrokerUrl);
@@ -150,6 +157,9 @@ class ConsumerImpl : public ConsumerImplBase {
     void beforeConnectionChange(ClientConnection& cnx) override;
     void onNegativeAcksSend(const std::set<MessageId>& messageIds);
 
+    void doImmediateAck(const MessageId& msgId, const ResultCallback& callback, CommandAck_AckType ackType);
+    void doImmediateAck(const std::set<MessageId>& msgIds, const ResultCallback& callback);
+
    protected:
     // overrided methods from HandlerBase
     Future<Result, bool> connectionOpened(const ClientConnectionPtr& cnx) override;
@@ -184,7 +194,7 @@ class ConsumerImpl : public ConsumerImplBase {
                                                 const BitSet& ackSet, int redeliveryCount);
     bool isPriorBatchIndex(int32_t idx);
     bool isPriorEntryIndex(int64_t idx);
-    void brokerConsumerStatsListener(Result, BrokerConsumerStatsImpl, BrokerConsumerStatsCallback);
+    void brokerConsumerStatsListener(Result, BrokerConsumerStatsImpl, const BrokerConsumerStatsCallback&);
 
     bool decryptMessageIfNeeded(const ClientConnectionPtr& cnx, const proto::CommandMessage& msg,
                                 const proto::MessageMetadata& metadata, SharedBuffer& payload);
@@ -199,7 +209,7 @@ class ConsumerImpl : public ConsumerImplBase {
     void trackMessage(const MessageId& messageId);
     void internalGetLastMessageIdAsync(const BackoffPtr& backoff, TimeDuration remainTime,
                                        const DeadlineTimerPtr& timer,
-                                       BrokerGetLastMessageIdCallback callback);
+                                       const BrokerGetLastMessageIdCallback& callback);
 
     void clearReceiveQueue();
     using SeekArg = boost::variant<uint64_t, MessageId>;
@@ -213,9 +223,9 @@ class ConsumerImpl : public ConsumerImplBase {
         return os;
     }
 
-    void seekAsyncInternal(long requestId, SharedBuffer seek, const SeekArg& seekArg,
-                           ResultCallback callback);
-    void processPossibleToDLQ(const MessageId& messageId, ProcessDLQCallBack cb);
+    void seekAsyncInternal(long requestId, const SharedBuffer& seek, const SeekArg& seekArg,
+                           const ResultCallback& callback);
+    void processPossibleToDLQ(const MessageId& messageId, const ProcessDLQCallBack& cb);
 
     std::mutex mutexForReceiveWithZeroQueueSize;
     const ConsumerConfiguration config_;
@@ -235,7 +245,7 @@ class ConsumerImpl : public ConsumerImplBase {
     std::queue<ReceiveCallback> pendingReceives_;
     std::atomic_int availablePermits_;
     const int receiverQueueRefillThreshold_;
-    uint64_t consumerId_;
+    const uint64_t consumerId_;
     const std::string consumerStr_;
     int32_t partitionIndex_ = -1;
     Promise<Result, ConsumerImplBaseWeakPtr> consumerCreatedPromise_;
@@ -244,7 +254,7 @@ class ConsumerImpl : public ConsumerImplBase {
     UnAckedMessageTrackerPtr unAckedMessageTrackerPtr_;
     BrokerConsumerStatsImpl brokerConsumerStats_;
     std::shared_ptr<NegativeAcksTracker> negativeAcksTracker_;
-    AckGroupingTrackerPtr ackGroupingTrackerPtr_;
+    const AckGroupingTrackerPtr ackGroupingTrackerPtr_;
 
     MessageCryptoPtr msgCrypto_;
     const bool readCompacted_;
@@ -264,6 +274,7 @@ class ConsumerImpl : public ConsumerImplBase {
     Synchronized<MessageId> seekMessageId_{MessageId::earliest()};
     std::atomic<bool> hasSoughtByTimestamp_{false};
 
+    bool hasSoughtByTimestamp() const { return hasSoughtByTimestamp_.load(std::memory_order_acquire); }
     bool duringSeek() const { return seekStatus_ != SeekStatus::NOT_STARTED; }
 
     class ChunkedMessageCtx {
@@ -277,9 +288,9 @@ class ConsumerImpl : public ConsumerImplBase {
         ChunkedMessageCtx(const ChunkedMessageCtx&) = delete;
         // Here we don't use =default to be compatible with GCC 4.8
         ChunkedMessageCtx(ChunkedMessageCtx&& rhs) noexcept
-            : totalChunks_(rhs.totalChunks_),
-              chunkedMsgBuffer_(std::move(rhs.chunkedMsgBuffer_)),
-              chunkedMessageIds_(std::move(rhs.chunkedMessageIds_)) {}
+            : totalChunks_(rhs.totalChunks_), chunkedMsgBuffer_(std::move(rhs.chunkedMsgBuffer_)) {
+            std::swap(chunkedMessageIds_, rhs.chunkedMessageIds_);
+        }
 
         bool validateChunkId(int chunkId) const noexcept { return chunkId == numChunks(); }
 
@@ -295,7 +306,11 @@ class ConsumerImpl : public ConsumerImplBase {
 
         const std::vector<MessageId>& getChunkedMessageIds() const noexcept { return chunkedMessageIds_; }
 
-        std::vector<MessageId> moveChunkedMessageIds() noexcept { return std::move(chunkedMessageIds_); }
+        std::vector<MessageId> moveChunkedMessageIds() noexcept {
+            std::vector<MessageId> result;
+            result.swap(chunkedMessageIds_);
+            return result;
+        }
 
         long getReceivedTimeMs() const noexcept { return receivedTimeMs_; }
 
@@ -333,9 +348,12 @@ class ConsumerImpl : public ConsumerImplBase {
     std::atomic_bool expireChunkMessageTaskScheduled_{false};
 
     ConsumerInterceptorsPtr interceptors_;
+    const std::shared_ptr<std::atomic<uint64_t>> requestIdGenerator_;
+
+    uint64_t newRequestId() const { return (*requestIdGenerator_)++; }
 
     void triggerCheckExpiredChunkedTimer();
-    void discardChunkMessages(std::string uuid, MessageId messageId, bool autoAck);
+    void discardChunkMessages(const std::string& uuid, const MessageId& messageId, bool autoAck);
 
     /**
      * Process a chunk. If the chunk is the last chunk of a message, concatenate all buffered chunks into the
@@ -372,6 +390,11 @@ class ConsumerImpl : public ConsumerImplBase {
         }
     }
 
+    void doImmediateAck(const ClientConnectionPtr& cnx, const MessageId& msgId, CommandAck_AckType ackType,
+                        const ResultCallback& callback);
+    void doImmediateAck(const ClientConnectionPtr& cnx, const std::set<MessageId>& msgIds,
+                        const ResultCallback& callback);
+
     friend class PulsarFriend;
     friend class MultiTopicsConsumerImpl;
 
