@@ -62,7 +62,7 @@ class MessageCrypto {
      * @return ResultOk if succeeded
      *
      */
-    Result addPublicKeyCipher(const std::set<std::string>& keyNames, const CryptoKeyReaderPtr keyReader);
+    Result addPublicKeyCipher(const std::set<std::string>& keyNames, const CryptoKeyReaderPtr& keyReader);
 
     /*
      * Remove a key <p> Remove the key identified by the keyName from the list of keys.<p>
@@ -84,7 +84,7 @@ class MessageCrypto {
      *
      * @return true if success
      */
-    bool encrypt(const std::set<std::string>& encKeys, const CryptoKeyReaderPtr keyReader,
+    bool encrypt(const std::set<std::string>& encKeys, const CryptoKeyReaderPtr& keyReader,
                  proto::MessageMetadata& msgMetadata, SharedBuffer& payload, SharedBuffer& encryptedPayload);
 
     /*
@@ -98,13 +98,13 @@ class MessageCrypto {
      * @return true if success
      */
     bool decrypt(const proto::MessageMetadata& msgMetadata, SharedBuffer& payload,
-                 const CryptoKeyReaderPtr keyReader, SharedBuffer& decryptedPayload);
+                 const CryptoKeyReaderPtr& keyReader, SharedBuffer& decryptedPayload);
 
    private:
     typedef std::unique_lock<std::mutex> Lock;
     std::mutex mutex_;
 
-    int dataKeyLen_;
+    size_t dataKeyLen_;
     boost::scoped_array<unsigned char> dataKey_;
 
     int tagLen_;
@@ -125,13 +125,17 @@ class MessageCrypto {
 
     EVP_MD_CTX* mdCtx_;
 
-    RSA* loadPublicKey(std::string& pubKeyStr);
-    RSA* loadPrivateKey(std::string& privateKeyStr);
+    EVP_PKEY* loadPublicKey(std::string& pubKeyStr);
+    EVP_PKEY* loadPrivateKey(std::string& privateKeyStr);
+    bool rsaDecrypt(EVP_PKEY_CTX* ctx, const std::string& in, boost::scoped_array<unsigned char>& out,
+                    size_t& outLen);
+    bool rsaEncrypt(EVP_PKEY_CTX* ctx, boost::scoped_array<unsigned char>& in, size_t inLen,
+                    boost::scoped_array<unsigned char>& out, size_t& outLen);
     bool getDigest(const std::string& keyName, const void* input, unsigned int inputLen,
                    unsigned char keyDigest[], unsigned int& digestLen);
     void removeExpiredDataKey();
 
-    Result addPublicKeyCipher(const std::string& keyName, const CryptoKeyReaderPtr keyReader);
+    Result addPublicKeyCipher(const std::string& keyName, const CryptoKeyReaderPtr& keyReader);
 
     bool decryptDataKey(const proto::EncryptionKeys& encKeys, const CryptoKeyReader& keyReader);
     bool decryptData(const std::string& dataKeySecret, const proto::MessageMetadata& msgMetadata,
