@@ -19,19 +19,24 @@
 
 #include "NegativeAcksTracker.h"
 
+#include <cstdint>
 #include <functional>
 #include <set>
+#include <utility>
 
 #include "ClientImpl.h"
 #include "ConsumerImpl.h"
 #include "ExecutorService.h"
 #include "LogUtils.h"
 #include "MessageIdUtil.h"
+#include "pulsar/MessageBuilder.h"
+#include "pulsar/MessageId.h"
+#include "pulsar/MessageIdBuilder.h"
 DECLARE_LOG_OBJECT()
 
 namespace pulsar {
 
-NegativeAcksTracker::NegativeAcksTracker(ClientImplPtr client, ConsumerImpl &consumer,
+NegativeAcksTracker::NegativeAcksTracker(const ClientImplPtr &client, ConsumerImpl &consumer,
                                          const ConsumerConfiguration &conf)
     : consumer_(consumer),
       timerInterval_(0),
@@ -41,6 +46,7 @@ NegativeAcksTracker::NegativeAcksTracker(ClientImplPtr client, ConsumerImpl &con
     nackDelay_ =
         std::chrono::milliseconds(std::max(conf.getNegativeAckRedeliveryDelayMs(), MIN_NACK_DELAY_MILLIS));
     timerInterval_ = std::chrono::milliseconds((long)(nackDelay_.count() / 3));
+    nackPrecisionBit_ = conf.getNegativeAckPrecisionBitCnt();
     LOG_DEBUG("Created negative ack tracker with delay: " << nackDelay_.count() << " ms - Timer interval: "
                                                           << timerInterval_.count());
 }
@@ -50,7 +56,7 @@ void NegativeAcksTracker::scheduleTimer() {
         return;
     }
     std::weak_ptr<NegativeAcksTracker> weakSelf{shared_from_this()};
-    timer_->expires_from_now(timerInterval_);
+    timer_->expires_after(timerInterval_);
     timer_->async_wait([weakSelf](const ASIO_ERROR &ec) {
         if (auto self = weakSelf.lock()) {
             self->handleTimer(ec);
@@ -75,13 +81,22 @@ void NegativeAcksTracker::handleTimer(const ASIO_ERROR &ec) {
 
     auto now = Clock::now();
 
+    // The map is sorted by time, so we can exit immediately when we traverse to a time that does not match
     for (auto it = nackedMessages_.begin(); it != nackedMessages_.end();) {
-        if (it->second < now) {
-            messagesToRedeliver.insert(it->first);
-            it = nackedMessages_.erase(it);
-        } else {
-            ++it;
+        if (it->first > now) {
+            // We are done with all the messages that need to be redelivered
+            break;
         }
+
+        const auto &ledgerMap = it->second;
+        for (auto ledgerIt = ledgerMap.begin(); ledgerIt != ledgerMap.end(); ++ledgerIt) {
+            const auto &entrySet = ledgerIt->second;
+            for (auto setIt = entrySet.begin(); setIt != entrySet.end(); ++setIt) {
+                messagesToRedeliver.insert(
+                    MessageIdBuilder().ledgerId(ledgerIt->first).entryId(*setIt).build());
+            }
+        }
+        it = nackedMessages_.erase(it);
     }
     lock.unlock();
 
@@ -92,14 +107,27 @@ void NegativeAcksTracker::handleTimer(const ASIO_ERROR &ec) {
     scheduleTimer();
 }
 
+std::chrono::steady_clock::time_point trimLowerBit(const std::chrono::steady_clock::time_point &tp,
+                                                   int bits) {
+    // get origin timestamp in nanoseconds
+    auto timestamp = std::chrono::duration_cast<std::chrono::nanoseconds>(tp.time_since_epoch()).count();
+
+    // trim lower bits
+    auto trimmedTimestamp = timestamp & (~((1LL << bits) - 1));
+
+    return std::chrono::steady_clock::time_point(std::chrono::nanoseconds(trimmedTimestamp));
+}
+
 void NegativeAcksTracker::add(const MessageId &m) {
     auto msgId = discardBatch(m);
     auto now = Clock::now();
 
     {
         std::lock_guard<std::mutex> lock{mutex_};
+        auto trimmedTimestamp = trimLowerBit(now + nackDelay_, nackPrecisionBit_);
+        // If the timestamp is already in the map, we can just add the message to the existing entry
         // Erase batch id to group all nacks from same batch
-        nackedMessages_[msgId] = now + nackDelay_;
+        nackedMessages_[trimmedTimestamp][msgId.ledgerId()].insert(msgId.entryId());
     }
 
     scheduleTimer();
@@ -107,8 +135,7 @@ void NegativeAcksTracker::add(const MessageId &m) {
 
 void NegativeAcksTracker::close() {
     closed_ = true;
-    ASIO_ERROR ec;
-    timer_->cancel(ec);
+    cancelTimer(*timer_);
     std::lock_guard<std::mutex> lock(mutex_);
     nackedMessages_.clear();
 }
