@@ -36,8 +36,8 @@ namespace pulsar {
 
 const std::string PartitionedProducerImpl::PARTITION_NAME_SUFFIX = "-partition-";
 
-PartitionedProducerImpl::PartitionedProducerImpl(ClientImplPtr client, const TopicNamePtr topicName,
-                                                 const unsigned int numPartitions,
+PartitionedProducerImpl::PartitionedProducerImpl(const ClientImplPtr& client, const TopicNamePtr& topicName,
+                                                 unsigned int numPartitions,
                                                  const ProducerConfiguration& config,
                                                  const ProducerInterceptorsPtr& interceptors)
     : client_(client),
@@ -79,7 +79,7 @@ MessageRoutingPolicyPtr PartitionedProducerImpl::getMessageRouter() {
     }
 }
 
-PartitionedProducerImpl::~PartitionedProducerImpl() { shutdown(); }
+PartitionedProducerImpl::~PartitionedProducerImpl() { internalShutdown(); }
 // override
 const std::string& PartitionedProducerImpl::getTopic() const { return topic_; }
 
@@ -144,9 +144,8 @@ void PartitionedProducerImpl::start() {
     }
 }
 
-void PartitionedProducerImpl::handleSinglePartitionProducerCreated(Result result,
-                                                                   ProducerImplBaseWeakPtr producerWeakPtr,
-                                                                   unsigned int partitionIndex) {
+void PartitionedProducerImpl::handleSinglePartitionProducerCreated(
+    Result result, const ProducerImplBaseWeakPtr& producerWeakPtr, unsigned int partitionIndex) {
     // to indicate, we are doing cleanup using closeAsync after producer create
     // has failed and the invocation of closeAsync is not from client
     const auto numPartitions = getNumPartitionsWithLock();
@@ -234,9 +233,9 @@ void PartitionedProducerImpl::sendAsync(const Message& msg, SendCallback callbac
     } else {
         // Wrapping the callback into a lambda has overhead, so we check if the producer is ready first
         producer->getProducerCreatedFuture().addListener(
-            [msg, callback](Result result, ProducerImplBaseWeakPtr weakProducer) {
+            [msg, callback](Result result, const ProducerImplBaseWeakPtr& weakProducer) {
                 if (result == ResultOk) {
-                    weakProducer.lock()->sendAsync(msg, std::move(callback));
+                    weakProducer.lock()->sendAsync(msg, callback);
                 } else if (callback) {
                     callback(result, {});
                 }
@@ -245,7 +244,9 @@ void PartitionedProducerImpl::sendAsync(const Message& msg, SendCallback callbac
 }
 
 // override
-void PartitionedProducerImpl::shutdown() {
+void PartitionedProducerImpl::shutdown() { internalShutdown(); }
+
+void PartitionedProducerImpl::internalShutdown() {
     cancelTimers();
     interceptors_->close();
     auto client = client_.lock();
@@ -285,7 +286,7 @@ int64_t PartitionedProducerImpl::getLastSequenceId() const {
 void PartitionedProducerImpl::closeAsync(CloseCallback originalCallback) {
     auto closeCallback = [this, originalCallback](Result result) {
         if (result == ResultOk) {
-            shutdown();
+            internalShutdown();
         }
         if (originalCallback) {
             originalCallback(result);
@@ -331,9 +332,8 @@ void PartitionedProducerImpl::closeAsync(CloseCallback originalCallback) {
 }
 
 // `callback` is a wrapper of user provided callback, it's not null and will call `shutdown()`
-void PartitionedProducerImpl::handleSinglePartitionProducerClose(Result result,
-                                                                 const unsigned int partitionIndex,
-                                                                 CloseCallback callback) {
+void PartitionedProducerImpl::handleSinglePartitionProducerClose(Result result, unsigned int partitionIndex,
+                                                                 const CloseCallback& callback) {
     if (state_ == Failed) {
         // we should have already notified the client by callback
         return;
@@ -421,7 +421,7 @@ void PartitionedProducerImpl::flushAsync(FlushCallback callback) {
 
 void PartitionedProducerImpl::runPartitionUpdateTask() {
     auto weakSelf = weak_from_this();
-    partitionsUpdateTimer_->expires_from_now(partitionsUpdateInterval_);
+    partitionsUpdateTimer_->expires_after(partitionsUpdateInterval_);
     partitionsUpdateTimer_->async_wait([weakSelf](const ASIO_ERROR& ec) {
         auto self = weakSelf.lock();
         if (self) {
@@ -524,8 +524,7 @@ uint64_t PartitionedProducerImpl::getNumberOfConnectedProducer() {
 
 void PartitionedProducerImpl::cancelTimers() noexcept {
     if (partitionsUpdateTimer_) {
-        ASIO_ERROR ec;
-        partitionsUpdateTimer_->cancel(ec);
+        cancelTimer(*partitionsUpdateTimer_);
     }
 }
 
