@@ -65,7 +65,7 @@ class PULSAR_PUBLIC TopicName : public ServiceUnitId {
     NamespaceNamePtr getNamespaceName();
     int getPartitionIndex() const noexcept { return partition_; }
     static std::shared_ptr<TopicName> get(const std::string& topicName);
-    bool operator==(const TopicName& other);
+    bool operator==(const TopicName& other) const;
     static std::string getEncodedName(const std::string& nameBeforeEncoding);
     static std::string removeDomain(const std::string& topicName);
     static bool containsDomain(const std::string& topicName);
