@@ -25,6 +25,7 @@
 #include <atomic>
 #include <chrono>
 #include <ctime>
+#include <future>
 #include <map>
 #include <mutex>
 #include <set>
@@ -61,7 +62,7 @@ namespace pulsar {
 
 class ConsumerStateEventListener : public ConsumerEventListener {
    public:
-    ConsumerStateEventListener(std::string name) { name_ = name; }
+    ConsumerStateEventListener(std::string name) { name_ = std::move(name); }
 
     void becameActive(Consumer consumer, int partitionId) override {
         LOG_INFO("Received consumer active event, partitionId:" << partitionId << ", name: " << name_);
@@ -1499,4 +1500,99 @@ TEST(ConsumerTest, testReconnectWhenFirstConnectTimedOut) {
     client.close();
 }
 
+TEST(ConsumerTest, testDuplicatedTopics) {
+    Client client{lookupUrl};
+    auto topicPrefix = "consumer-test-duplicated-topics" + std::to_string(time(nullptr));
+    std::array<std::string, 3> uniqueTopics{topicPrefix + "0", topicPrefix + "1", topicPrefix + "2"};
+    std::vector<std::string> topics{uniqueTopics[1], uniqueTopics[0], uniqueTopics[2], uniqueTopics[1],
+                                    uniqueTopics[2]};
+
+    Consumer consumer;
+    ASSERT_EQ(ResultOk, client.subscribe(topics, "sub", consumer));
+
+    for (size_t i = 0; i < uniqueTopics.size(); i++) {
+        Producer producer;
+        ASSERT_EQ(ResultOk, client.createProducer(topics[i], producer));
+        ASSERT_EQ(ResultOk, producer.send(MessageBuilder().setContent("msg-" + std::to_string(i)).build()));
+        Message msg;
+        ASSERT_EQ(ResultOk, consumer.receive(msg, 3000));
+        ASSERT_EQ("msg-" + std::to_string(i), msg.getDataAsString());
+    }
+}
+
+TEST(ConsumerTest, testConsumerListenerShouldNotSegfaultAfterClose) {
+    Client client(lookupUrl);
+
+    const int MSG_COUNT = 100;
+    std::string topicName = "persistent://public/default/my-topic-" + std::to_string(time(nullptr));
+
+    // 1. Create producer send 100 msgs
+    Producer producer;
+    ProducerConfiguration producerConfig;
+    producerConfig.setBatchingEnabled(false);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConfig, producer));
+    for (int i = 0; i < MSG_COUNT; ++i) {
+        std::string msg = "my-message-" + std::to_string(i);
+        Message message = MessageBuilder().setContent(msg).build();
+        ASSERT_EQ(ResultOk, producer.send(message));
+    }
+
+    // 2. Create consumer with listener
+    Consumer consumer;
+    ConsumerConfiguration consumerConfig;
+    consumerConfig.setSubscriptionInitialPosition(InitialPositionEarliest);
+    Latch latchFirstReceiveMsg(1);
+    Latch latchAfterClosed(1);
+
+    std::promise<std::vector<Result>> ackResultsPromise;
+    consumerConfig.setMessageListener([&latchFirstReceiveMsg, &latchAfterClosed, &ackResultsPromise](
+                                          Consumer consumer, const Message& msg) {
+        latchFirstReceiveMsg.countdown();
+        LOG_INFO("Consume message: " << msg.getDataAsString());
+        latchAfterClosed.wait();
+        std::vector<Result> results(3);
+        results[0] = consumer.acknowledge(msg);
+        results[1] = consumer.acknowledgeCumulative(msg);
+        results[2] = consumer.acknowledge(std::vector<MessageId>{msg.getMessageId()});
+        ackResultsPromise.set_value(results);
+    });
+    auto result = client.subscribe(topicName, "test-sub", consumerConfig, consumer);
+    ASSERT_EQ(ResultOk, result);
+
+    // 3. wait first message consumed in listener and then close consumer.
+    latchFirstReceiveMsg.wait();
+    ASSERT_EQ(ResultOk, consumer.close());
+    latchAfterClosed.countdown();
+    const auto ackResults = ackResultsPromise.get_future().get();
+    ASSERT_EQ(3, ackResults.size());
+    for (size_t i = 0; i < ackResults.size(); i++) {
+        ASSERT_EQ(ResultAlreadyClosed, ackResults[i]) << "ack result[" << i << "] " << ackResults[i];
+    }
+
+    ASSERT_EQ(ResultOk, producer.close());
+    ASSERT_EQ(ResultOk, client.close());
+}
+
+TEST(ConsumerTest, testCloseAfterSeek) {
+    const auto topic = "test-close-after-seek-" + std::to_string(time(nullptr));
+    const auto subscription = "sub";
+    Client client(lookupUrl);
+    Consumer consumer;
+    ASSERT_EQ(ResultOk, client.subscribe(topic, subscription, consumer));
+    ASSERT_EQ(ResultOk, consumer.seek(TimeUtils::currentTimeMillis()));
+    consumer.closeAsync(nullptr);
+
+    // Test the previous consumer will be closed even after seek is done, at the moment the connection might
+    // not be established.
+    ASSERT_EQ(ResultOk, client.subscribe(topic, subscription, consumer));
+
+    // Test creating a consumer from a different client should also work for this case
+    Client anotherClient(lookupUrl);
+    consumer.closeAsync(nullptr);
+    ASSERT_EQ(ResultOk, anotherClient.subscribe(topic, subscription, consumer));
+
+    client.close();
+    anotherClient.close();
+}
+
 }  // namespace pulsar
