@@ -41,7 +41,7 @@ static const std::string adminUrl = "http://localhost:8080/";
 
 class ReaderTest : public ::testing::TestWithParam<bool> {
    public:
-    void initTopic(std::string topicName) {
+    void initTopic(const std::string& topicName) {
         if (isMultiTopic_) {
             // call admin api to make it partitioned
             std::string url = adminUrl + "admin/v2/persistent/public/default/" + topicName + "/partitions";
@@ -67,7 +67,9 @@ TEST_P(ReaderTest, testSimpleReader) {
     ASSERT_EQ(ResultOk, client.createReader(topicName, MessageId::earliest(), readerConf, reader));
 
     Producer producer;
-    ASSERT_EQ(ResultOk, client.createProducer(topicName, producer));
+    ProducerConfiguration producerConf;
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     for (int i = 0; i < 10; i++) {
         std::string content = "my-message-" + std::to_string(i);
@@ -142,7 +144,9 @@ TEST_P(ReaderTest, testReaderAfterMessagesWerePublished) {
     initTopic(topicName);
 
     Producer producer;
-    ASSERT_EQ(ResultOk, client.createProducer(topicName, producer));
+    ProducerConfiguration producerConf;
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     for (int i = 0; i < 10; i++) {
         std::string content = "my-message-" + std::to_string(i);
@@ -176,7 +180,9 @@ TEST_P(ReaderTest, testMultipleReaders) {
     initTopic(topicName);
 
     Producer producer;
-    ASSERT_EQ(ResultOk, client.createProducer(topicName, producer));
+    ProducerConfiguration producerConf;
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     for (int i = 0; i < 10; i++) {
         std::string content = "my-message-" + std::to_string(i);
@@ -223,7 +229,9 @@ TEST_P(ReaderTest, testReaderOnLastMessage) {
     initTopic(topicName);
 
     Producer producer;
-    ASSERT_EQ(ResultOk, client.createProducer(topicName, producer));
+    ProducerConfiguration producerConf;
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     for (int i = 0; i < 10; i++) {
         std::string content = "my-message-" + std::to_string(i);
@@ -263,7 +271,9 @@ TEST_P(ReaderTest, testReaderOnSpecificMessage) {
     initTopic(topicName);
 
     Producer producer;
-    ASSERT_EQ(ResultOk, client.createProducer(topicName, producer));
+    ProducerConfiguration producerConf;
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
+    ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     for (int i = 0; i < 10; i++) {
         std::string content = "my-message-" + std::to_string(i);
@@ -459,6 +469,7 @@ TEST_P(ReaderTest, testReaderReachEndOfTopicMessageWithoutBatches) {
     Producer producer;
     ProducerConfiguration producerConf;
     producerConf.setBatchingEnabled(false);
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
     ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
     // 2. create reader, and expect hasMessageAvailable return false since no message produced.
@@ -634,6 +645,7 @@ TEST_P(ReaderTest, testHasMessageAvailableWhenCreated) {
 
     ProducerConfiguration producerConf;
     producerConf.setBatchingMaxMessages(3);
+    producerConf.setPartitionsRoutingMode(ProducerConfiguration::UseSinglePartition);
     Producer producer;
     ASSERT_EQ(ResultOk, client.createProducer(topicName, producerConf, producer));
 
@@ -885,6 +897,16 @@ TEST_F(ReaderSeekTest, testHasMessageAvailableAfterSeekTimestamp) {
         createReader(reader, msgId);
         ASSERT_EQ(ResultOk, reader.seek(timestampBeforeSend));
         EXPECT_HAS_MESSAGE_AVAILABLE(reader, true);
+
+        bool hasMessageAvailable;
+        while (true) {
+            ASSERT_EQ(ResultOk, reader.hasMessageAvailable(hasMessageAvailable));
+            if (!hasMessageAvailable) {
+                break;
+            }
+            Message msg;
+            ASSERT_EQ(ResultOk, reader.readNext(msg, 3000));
+        }
     }
 
     // Test `hasMessageAvailableAsync` will complete immediately if the incoming message queue is non-empty
@@ -919,7 +941,7 @@ TEST_F(ReaderSeekTest, testSeekInclusiveChunkMessage) {
     MessageId secondMsgId;
     ASSERT_EQ(ResultOk, producer.send(MessageBuilder().setContent(largeValue).build(), secondMsgId));
 
-    auto assertStartMessageId = [&](bool inclusive, MessageId expectedMsgId) {
+    auto assertStartMessageId = [&](bool inclusive, const MessageId& expectedMsgId) {
         Reader reader;
         ReaderConfiguration readerConf;
         readerConf.setStartMessageIdInclusive(inclusive);
