@@ -96,21 +96,21 @@ TEST(SynchronizedHashMapTest, testForEach) {
     m.clear();
     int result = 0;
     values.clear();
-    m.forEachValue([&values](int value, SharedFuture) { values.emplace_back(value); },
+    m.forEachValue([&values](int value, const SharedFuture&) { values.emplace_back(value); },
                    [&result] { result = 1; });
     ASSERT_TRUE(values.empty());
     ASSERT_EQ(result, 1);
 
     ASSERT_EQ(m.putIfAbsent(1, 100), boost::none);
     ASSERT_EQ(m.putIfAbsent(1, 101), boost::optional<int>(100));
-    m.forEachValue([&values](int value, SharedFuture) { values.emplace_back(value); },
+    m.forEachValue([&values](int value, const SharedFuture&) { values.emplace_back(value); },
                    [&result] { result = 2; });
     ASSERT_EQ(values, (std::vector<int>({100})));
     ASSERT_EQ(result, 1);
 
     m.put(1, 102);
     values.clear();
-    m.forEachValue([&values](int value, SharedFuture) { values.emplace_back(value); },
+    m.forEachValue([&values](int value, const SharedFuture&) { values.emplace_back(value); },
                    [&result] { result = 2; });
     ASSERT_EQ(values, (std::vector<int>({102})));
     ASSERT_EQ(result, 1);
@@ -118,7 +118,7 @@ TEST(SynchronizedHashMapTest, testForEach) {
     values.clear();
     ASSERT_EQ(m.putIfAbsent(2, 200), boost::none);
     ASSERT_EQ(m.putIfAbsent(2, 201), boost::optional<int>(200));
-    m.forEachValue([&values](int value, SharedFuture) { values.emplace_back(value); },
+    m.forEachValue([&values](int value, const SharedFuture&) { values.emplace_back(value); },
                    [&result] { result = 2; });
     std::sort(values.begin(), values.end());
     ASSERT_EQ(values, (std::vector<int>({102, 200})));
